# ConfDialog.py
#
# Copyright (C) 2008 - Teemu Kallio <teemu.kallio@cs.helsinki.fi>
#				2009-2010 - Floreal Morandat <morandat AT lirmm DOT fr>
#					2012-2013 Mehmet Giritli <mehmet@giritli.eu>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

from gi.repository import Gtk, Gio, Gdk, GdkPixbuf, Gst
import Conf

STOCK_IMAGE = "stock-equalizer-button"

class ConfDialog(object):
	def __init__(self, glade_file, conf, eq, plugin):
		self.plugin = plugin
		self.eq = eq
		self.conf = conf
		gladexml = Gtk.Builder()
		gladexml.add_from_file(glade_file)
	
		self.dialog = gladexml.get_object('preferences_dialog')
		self.dialog.connect("response", self.dialog_response)
		self.dialog.connect("destroy", self.on_destroy)
		self.dialog.connect("close", self.on_destroy)

		self.modified = 0
		self.init_done = 0

		self.box = gladexml.get_object("presetchooser")
			
		self.delete_btn = gladexml.get_object("deletebutton1")
		self.reset_btn = gladexml.get_object("resetbutton1")
		self.reset_all_btn = gladexml.get_object("resetbutton2")
		self.check = gladexml.get_object("autocheck1")

		self.check.set_active(conf.get_auto_select())
		self.check.connect("toggled", self.manage_auto_select)

		# workarounds
		# see https://bugzilla.gnome.org/show_bug.cgi?id=650369#c4
		self.box.set_entry_text_column(0)
		self.box.set_id_column(1)
		
		self.read_presets()
		self.box.connect("changed", self.preset_change)

		self.bands = []
		for i in range(0,10):
			self.bands.append(gladexml.get_object("b" + `i`))
			self.bands[i].connect("value_changed", self.slider_changed)
		self.update_bands()
		conf.apply_settings(eq)

	def cleanup(self):
		self.plugin.shell.props.ui_manager.remove_action_group(self.action_group)
		self.plugin.shell.props.ui_manager.remove_ui(self.ui)
		
	def on_destroy(self, dialog):
		dialog.hide()
		self.__init__(self.plugin.glade_f, self.conf, self.eq, self.plugin)
		return True	
        
	def read_presets(self):
		box = self.box
		conf = self.conf
		if box:
			box.get_model().clear()

		current_preset = conf.preset
		custom_presets = sorted(conf.get_custom_presets())
		s_presets = sorted(Gst.Preset.get_preset_names(self.eq))
		i = 1
		box.append_text("default")
		box.set_active(0)
		for preset in s_presets + custom_presets:
			box.append_text(preset)
			if (preset == current_preset):
				box.set_active(i)
			i += 1

	def update_bands(self):
		for i in range(0, 10):
			self.bands[i].set_value(self.conf.config[i])

	def get_dialog(self):
		return self.dialog

	def dialog_response(self, dialog, response):
		if (response == -4):
			if (self.init_done == 1 and self.modified == 1):
				self.get_custom_preset_details()
			dialog.hide()
			
		if (response == -6):
			self.conf.reset_all(self.eq)
			self.box.set_active(0)
		
		if (response == -8 or response == -6):
			if self.box.get_active_text() == "default":
				self.conf.config = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
			else:
				Gst.Preset.load_preset(self.eq, self.box.get_active_text())
				self.conf.config = list(self.eq.get_property('band' + str(i)) for i in range(0,10))
				
			self.update_bands()

		if (response == -10):
			self.conf.delete_custom_preset(self.box.get_active_text())
			self.conf.preset = "default"
			self.read_presets()

	def get_custom_preset_details(self):
		modification_msg = "You modified a preset. Would you like to save it?"
		inputDialog = Gtk.MessageDialog(self.dialog, Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT, Gtk.MessageType.QUESTION, Gtk.ButtonsType.YES_NO, modification_msg)

		contentBox = inputDialog.get_content_area()
		userEntry = Gtk.Entry()
		userEntry.set_max_length(50)
		userEntry.set_size_request(250,0)
		userEntry.set_text(self.conf.preset)
		contentBox.pack_end(userEntry, False, False, 0)

		actionBox = inputDialog.get_action_area()
		ok_button = actionBox.get_children()[0]

		userEntry.set_activates_default(True)
		inputDialog.set_default(ok_button)

		userEntry.connect("changed", self.check_input, ok_button)

		inputDialog.show_all()
		response = inputDialog.run()

		text = userEntry.get_text()
		inputDialog.destroy()
		if response == Gtk.ResponseType.YES:
			text_after_check = self.conf.add_custom_preset(text, self.eq)
			self.conf.preset = text_after_check
			self.conf.write_settings()
		self.modified = 0
			
	def check_input(self, entry, button):
		if (entry.get_text().strip() == ""):
			button.set_sensitive(False)
		else:
			button.set_sensitive(True)

	def preset_change(self, entry):
		new_preset = entry.get_active_text()

		if new_preset in Gst.Preset.get_preset_names(self.eq) + ['default']:
			self.delete_btn.set_sensitive(False)
			self.reset_btn.set_sensitive(True)
			self.reset_all_btn.set_sensitive(True)
		else:
			self.delete_btn.set_sensitive(True)
			self.reset_btn.set_sensitive(False)
			self.reset_all_btn.set_sensitive(False)

		if (self.init_done == 1 and self.modified == 1):
			self.get_custom_preset_details()
			self.conf.preset = new_preset
			self.read_presets()

		self.init_done = 0
		if new_preset != '':
			self.conf.change_preset(entry.get_active_text(), self.eq)
			self.update_bands()
		self.init_done = 1

	def slider_changed(self, hscale):
		if self.init_done == 1:
			self.modified = 1
		eq = self.eq
		if eq == None:
			return
		i = self.bands.index(hscale)
		val = self.bands[i].get_value()
		self.conf.config[i] = val
		eq.set_property('band' + `i`, val)

	def manage_auto_select(self, check):
		if check.get_active():
			self.conf.set_auto_select(True)
		else:
			self.conf.set_auto_select(False)

	def add_ui(self, shell):
		plugin = self.plugin
		icon_factory = Gtk.IconFactory()
		pxbf = GdkPixbuf.Pixbuf.new_from_file(plugin.find_file("equalizer.svg"))
		icon_factory.add(STOCK_IMAGE, Gtk.IconSet.new_from_pixbuf(pxbf))
		icon_factory.add_default()

		action = Gtk.Action ('Equalize', 
				_('_Equalizer'), 
				_('10 Band Equalizer'),
				STOCK_IMAGE)
		action.connect ('activate', self.show_ui, shell)
		self.action_group = Gtk.ActionGroup ('EqualizerActionGroup')
		self.action_group.add_action (action)
		shell.props.ui_manager.insert_action_group(self.action_group, -1)
		ui_manager = shell.props.ui_manager
		self.ui = ui_manager.add_ui_from_file(plugin.find_file("equalizer-ui.xml"))

	def show_ui(self, shell, state):
		self.read_presets()
		self.get_dialog().present()
